package de.duehl.vocabulary.japanese.launcher.logic;

import de.duehl.basics.io.FileHelper;
import de.duehl.basics.system.SystemTools;
import de.duehl.basics.system.launcher.JarStarter;
import de.duehl.basics.system.launcher.NewestJarDeterminer;
import de.duehl.basics.version.Version;
import de.duehl.html.download.binary.SimpleBinaryFileDownloader;
import de.duehl.swing.ui.GuiTools;
import de.duehl.vocabulary.japanese.launcher.logic.launcher.LauncherSelfUpdateChecker;
import de.duehl.vocabulary.japanese.updater.launcher.SecretParameterForTrainerStart;
import de.duehl.vocabulary.japanese.updater.website.VocaluaryTrainerWebsiteConstants;
import de.duehl.vocabulary.japanese.updater.website.update.program.ProgramUpdaterTools;

import static de.duehl.vocabulary.japanese.updater.website.download.WebsiteFileContentDownloader.NO_DOWNLOAD_SUCCESS;

/**
 * Diese Klasse stellt den Launcher des Vokabeltrainers dar.
 *
 * @version 1.01     2024-11-11
 * @author Christian Dühl
 */

public class VocabularyTrainerLauncher {

    /** Der Anfang der Dateinamen der Jars mit dem Vokabeltrainer. */
    private final static String JAR_NAME_START = "de.duehl.vocabulary.japanese_";

    /**
     * Die Versionsnummer der neusten installierten Jar.
     *
     * Wenn sie nicht ermittelt werden konnte, ist sie leer.
     */
    private String newestVersionNumberInstalled;

    /**
     * Die Versionsnummer auf dem Server.
     *
     * Wenn sie nicht ermittelt werden konnte, ist sie leer.
     */
    private String versionNumberOnServer;

    /** Führt den Launcher aus. */
    public void launch() {
        boolean goOn = checkForSelfUpdate();
        if (goOn) {
            determineNewestVersionNumberInstalled();
            determineVersionNumberOnServer();
            determineWhatToPerhapsDownloadAndWhatToLauch();
        }
    }

    private boolean checkForSelfUpdate() {
        LauncherSelfUpdateChecker checker = new LauncherSelfUpdateChecker();
        checker.check();
        return checker.isGoOn();
    }

    private void determineNewestVersionNumberInstalled() {
        NewestJarDeterminer determiner = new NewestJarDeterminer(JAR_NAME_START);
        determiner.removeOldJars();
        determiner.determine();
        if (determiner.isJarFound()) {
            String newestJarFilename = determiner.getNewestJarFilename();
            newestVersionNumberInstalled = Version.extractVersionFromJarFilename(newestJarFilename,
                    JAR_NAME_START);
        }
        else {
            newestVersionNumberInstalled = "";
        }
    }

    private void determineVersionNumberOnServer() {
        versionNumberOnServer = ProgramUpdaterTools.downloadProgramVersionFromServer();
        if (versionNumberOnServer.equals(NO_DOWNLOAD_SUCCESS)) {
            versionNumberOnServer = "";
        }
    }

    private void determineWhatToPerhapsDownloadAndWhatToLauch() {
        boolean installedVersionAvailable = !newestVersionNumberInstalled.isEmpty();
        boolean versionOnWebsiteAvailable = !versionNumberOnServer.isEmpty();

        if (installedVersionAvailable && versionOnWebsiteAvailable) {          // beide vorhanden
            if (isVersionOnServerNewer()) {
                if (userWantsToDownloadNewerVersion()) {
                    downloadVersionFromServerAndStartIt();
                }
                else {
                    startNewestInstalledVersion();
                }
            }
            else {
                startNewestInstalledVersion();
            }
        }
        else if (installedVersionAvailable) {                     // nur lokal vorhanden
            startNewestInstalledVersion();
        }
        else if (versionOnWebsiteAvailable) {                     // nur auf dem Webserver vorhanden
            if (userWantsToDownloadMissingTrainerJar()) {
                downloadVersionFromServerAndStartIt();
            }
        }
        else {                                                    // keins vorhanden
            informUserWeHaveNothingToStart();
        }
    }

    private boolean isVersionOnServerNewer() {
        Version versionOnServer = new Version(versionNumberOnServer, "interessiert nicht");
        Version versionInstalled = new Version(newestVersionNumberInstalled, "interessiert nicht");
        return versionOnServer.isNewerThan(versionInstalled);
    }

    private boolean userWantsToDownloadNewerVersion() {
        String title = "Es gibt eine neuere Version auf dem Server";
        String message =  "Es gibt eine neuere Version auf dem Server:\n\n"
                + "Installierte Version: " + newestVersionNumberInstalled + "\n"
                + "Version auf dem Server: " + versionNumberOnServer + "\n\n"
                + "Möchten Sie die neue Version herunterladen?";
        return GuiTools.askUser(title, message);
    }

    private void downloadVersionFromServerAndStartIt() {
        String jarBareFilename = createLocalBareJarFilenameFromVersion(versionNumberOnServer);
        String jarFileUrl = VocaluaryTrainerWebsiteConstants.BASE_URL + jarBareFilename;
        String dir = SystemTools.getCurrentWorkingDirectory();
        String localJarFilename = FileHelper.concatPathes(dir, jarBareFilename);
        SimpleBinaryFileDownloader.downloadBinaryFile(jarFileUrl, localJarFilename);
        startJar(localJarFilename);
    }

    private static String createLocalBareJarFilenameFromVersion(String version) {
        return JAR_NAME_START + version + ".jar";
    }

    private void startJar(String localJarFilename) {
        JarStarter starter = new JarStarter(localJarFilename,
                SecretParameterForTrainerStart.SECRET_PARAMETER);
        starter.start();
        boolean success = starter.isSuccess();
        String command = starter.getCommand();
        if (!success) {
            String title = "Der Start des Jars hatte keinen Erfolg";
            String message =  "Der Start des Jars hatte keinen Erfolg:\n"
                    + "Abgeschickter Befehl: " + command;
            GuiTools.informUser(title, message);
        }

        /*
         * Es wäre interessant zu wissen, ob die einfach mit vollem Pfad gestartete Jar dann mit
         *     SystemTools.getCurrentWorkingDirectory();
         * auch wieder das richtige currentWorkingDirectory angeben  würde und nicht
         *     C:\\
         * oder sowas.
         *
         * Aber da der eigentliche Vokabeltrainer es nicht nutzt, ist das erstmal egal.
         */
    }

    private void startNewestInstalledVersion() {
        String jarBareFilename = createLocalBareJarFilenameFromVersion(newestVersionNumberInstalled);
        String dir = SystemTools.getCurrentWorkingDirectory();
        String localJarFilename = FileHelper.concatPathes(dir, jarBareFilename);
        startJar(localJarFilename);
    }

    private boolean userWantsToDownloadMissingTrainerJar() {
        String title = "Es gibt keine lokale Version, aber eine auf dem Server";
        String message =  "Es gibt keine lokale Version, aber eine auf dem Server:\n\n"
                + "Installierte Version: ---\n"
                + "Version auf dem Server: " + versionNumberOnServer + "\n\n"
                + "Möchten Sie die neue Version herunterladen?";
        return GuiTools.askUser(title, message);
    }

    private void informUserWeHaveNothingToStart() {
        String title = "Der Launcher kann den Vokabeltrainer nicht starten";
        String message =  "Es liegt keine installierte Jar des Vokabeltrainers vor und auch "
                + "von der Webseite\n"
                + "ist gerade keine erhältlich. Bitte probieren Sie es später noch einmal.";
        GuiTools.informUser(title, message);
    }

}
